// ### Libraries and globals

// This bot works by inspecting the front page of Google News. So we need
// to use `request` to make HTTP requests, `cheerio` to parse the page using
// a jQuery-like API, `underscore.deferred` for [promises](http://otaqui.com/blog/1637/introducing-javascript-promises-aka-futures-in-google-chrome-canary/),
// and `twit` as our Twitter API library.
var request = require('request');
var cheerio = require('cheerio');
var _ = require('underscore.deferred');
var cs = require('common-substrings').CommonSubstrings;
var Twit = require('twit');
var T = new Twit(require('./config.js'));
var wordfilter = require('wordfilter');
wordfilter.addWords(['rape']);

var genderProb = require('gender-probability');
genderProb.load();

var baseUrl = 'http://news.google.com';

// ### Utility Functions

// This function lets us call `pick()` on any array to get a random element from it.
Array.prototype.pick = function() {
  return this[Math.floor(Math.random()*this.length)];
};

// This function lets us call `pickRemove()` on any array to get a random element
// from it, then remove that element so we can't get it again.
Array.prototype.pickRemove = function() {
  var index = Math.floor(Math.random()*this.length);
  return this.splice(index,1)[0];
};

// This function analyzes two topics for gendered first names. If the topics have
// opposite probable genders then we are likely to be making a transphobic joke.
function isTransphobic(topic, newTopic) {
  var isTransphobic = false;
  var name = topic.split(/\s/)[0];
  var swapped = newTopic.split(/\s/)[0];
  var prob = genderProb.get(name);
  var probSwapped = genderProb.get(swapped);
  if (!prob.error && !probSwapped.error) {
    //console.log(name, prob.data.prob.gender, swapped, probSwapped.data.prob.gender);
    isTransphobic = (
        (prob.data.prob.gender === 'Male' && probSwapped.data.prob.gender === 'Female') ||
        (prob.data.prob.gender === 'Female' && probSwapped.data.prob.gender === 'Male')
      );
  }
  return isTransphobic;
}

function findCommonText(headlines) {
  //headlines = ['foo, bar baz', 'xxx "baz" xxx', 'yyy; baz'];
  var tree = new cs();
  tree.build(headlines);
  var avg = tree.weightByAverage();
  //console.log(avg.length);
  old_avg = avg;
  avg = avg.filter(function(el) {
    return el.name.trim().split(' ').length <= 2;
  });
  if (avg.length === 0) {
    avg = old_avg;
  }
  //console.log('avg: ', avg[0].name + ' ' + avg[0].weight);
  return avg[0].name.trim();
}
// ### Screen Scraping

// We pass this function a category code (see `tweet` below). We grab the Google News
// topics page for that category and load the html into `cheerio`. We parse the page for
// text from the links in the left-hand bar, which becomes a list of topics.
// For example, if we passed it 'e' for Entertainment, we might get: Miley Cyrus, Oscars,
// Kanye West, and so on.
function getTopics(category) {
  var topics = [];
  var dfd = new _.Deferred();
  request(baseUrl + '/news/section?pz=1&cf=all&topic=' + category, function (error, response, body) {
    if (!error && response.statusCode === 200) {
      var $ = cheerio.load(body);
      var groups = $('.esc-extension-wrapper');
      groups.each(function() {
        var headlines = [];
        // find lead article
        var lead = this.parent().find('.esc-lead-article-title-wrapper').find('.titletext').text()
        headlines.push(lead);
        this.find('.titletext').each(function() {
          headlines.push(this.text());
        });
        //console.log(headlines);
        var topic = {};
        topic.name = findCommonText(headlines).replace(/^\W*/,'').replace(/\W*$/,'');
        topic.headline = headlines.filter(function(el) {
            // filter out headlines that don't contain the topic as its own word
            var reg = new RegExp('\\b'+topic.name+'\\b');
            return el.match(reg) !== null;
          })
          .pick();
        // only add the topic if we have valid headlines (we found a valid replace target above)
        if (topic.headline && topic.headline.length > 1) {
          topics.push(topic);
        }
      });
      // filter out only topics that are proper nouns ('Capitalized Things' and not 'this stuff')
      topics = topics.filter(function(topic) {
        return topic.name.split(' ').filter(function(el) {
           return el[0] === el[0].toUpperCase(); }).length === topic.name.split(' ').length;
      });
      dfd.resolve(topics);
    }
    else {
      dfd.reject();
    }
  });
  // The function returns a promise, and the promise resolves to the array of topics.
  return dfd.promise();
}

// We pass this function a URL for a specific topic (for example:
// [Miley Cyrus](https://news.google.com/news/section?pz=1&cf=all&ned=us&hl=en&q=Miley%20Cyrus).
// We then get the page, feed the HTML to `cheerio`, and then pick a random headline
// from the page.
function getHeadline(url) {
  var dfd = new _.Deferred();
  request(url, function (error, response, body) {
    if (!error && response.statusCode === 200) {
      var $ = cheerio.load(body);
      var headlines = $('.titletext');
      // `pick()` doesn't work here because `headlines` isn't an array, so instead we use `cheerio`'s `eq` which
      // give us a matched element at a given index, and pass it a random number.
      var headline = headlines.eq(Math.floor(Math.random()*headlines.length)).text();
      dfd.resolve(headline);
    }
    else {
      dfd.reject();
    }
  });
  return dfd.promise();
}

// ### Tweeting

//      Category codes:
//      w:  world
//      n:  region
//      b:  business
//      tc: technology
//      e:  entertainment
//      s:  sports

// This is the core function that is called on a timer that initiates the @twoheadlines algorithm.
// First, we get our list of topics from the Google News sidebar. 
// Then we pick-and-remove a random topic from that list.
// Next we grab a random headline available for that topic.
// If the topic itself is in the headline itself, we replace it with a new topic. (For example,
// if `topic.name` is "Miley Cyrus" and `headline` is "Miley Cyrus Wins a Grammy", then we
// get a topic from a different category of news and fill in the blank for "______ Wins a Grammy".)
// If we're unable to find a headline where we can easily find/replace, we simply try again.
function tweet() {
  //console.log('hi');
  var categoryCodes = ['w', 'n', 'b', 'tc', 'e', 's'];
  getTopics(categoryCodes.pickRemove()).then(function(topics) {
    //console.log(topics);
    var topic = topics.pickRemove();
    //console.log('*',topic);
    getTopics(categoryCodes.pickRemove()).then(function(topics) {
      getTopics(categoryCodes.pickRemove()).then(function(topics) {
        var newTopic = topics.pick();
        //console.log(topic, newTopic);
        var newHeadline = topic.headline.replace(topic.name, newTopic.name);
        if (!isTransphobic(topic.name, newTopic.name) && !wordfilter.blacklisted(newHeadline)) {
          // This joke isn't transphobic so go ahead and tweet it
          console.log(newHeadline);
          T.post('statuses/update', { status: newHeadline }, function(err, reply) {
            if (err) {
              console.log('error:', err);
            }
            else {
              console.log('reply:', reply);
            }
          });
        }
        else {
          // This joke is transphobic, make a new one instead
          tweet();
        }
      });
    });
  });
}

// Tweets once on initialization.
tweet();
